'use strict';

const coAsyncLib = require('async');
const coFs = require('fs');

const coPathLib = require('../paths.js');

const Result = require('./result.js').Result;
const coValidateLib = require('./validate.js');
const coDwfAbstract = require('./abstract.js');

const coAbstractLib = require('./object/type/abstract.js');
const coTxtLib = require('./object/type/txt.js');
const coNamespaceLib = require('./object/type/nameSpace.js');
const coTaskchainLib = require('./object/type/taskChain.js');

function Container(ioCredential, ioOption, iaoFile) {
  this.loCredential = coValidateLib(ioCredential, 'object'); // Check - ioCredential schema validation
  this.loOption = coValidateLib(ioOption, 'object');
  this.laoFile = coValidateLib(iaoFile, 'object');

  let lnVersion = ioCredential[coDwfAbstract.cs_Credential_Key_Version];
  let lsVersionErrorMessage =
    'Failed, because the used version of the dwf-deploy does not support the current environment of the dwf module. ' +
    'The VCAP_SERVICES.dwf[<n>].credentials.' + coDwfAbstract.cs_Credential_Key_Version + ' = ' + lnVersion + ' is not supported. ' +
    'Solution: 1). Use a different version of the dwf-deploy module 2). Update the environment of the dwf module';
  coValidateLib(lnVersion, 'number', lsVersionErrorMessage);// Check - version
  if (lnVersion < 1) { // Check
    throw new Error(lsVersionErrorMessage);
  }

  let laoProvidersV1 = ioCredential[coDwfAbstract.cs_Credential_Key_Providers_v1];
  coValidateLib(laoProvidersV1, 'object', // Check - providers_v1
    'Failed, because the used version of the dwf-deploy does not support the current environment of the dwf module. ' +
    'The VCAP_SERVICES.dwf[<n>].credentials.' + coDwfAbstract.cs_Credential_Key_Providers_v1 + ' = ' + laoProvidersV1 + ' is not supported. ' +
    'Solution: 1). Use a different version of the dwf-deploy module 2). Update the environment of the dwf module');

  let loRepositoryV1 = ioCredential[coDwfAbstract.cs_Credential_Key_Repository_v1];
  coValidateLib(loRepositoryV1, 'object', // Check - repository_v1
    'Failed, because the used version of the dwf-deploy does not support the current environment of the dwf module. ' +
    'The VCAP_SERVICES.dwf[<n>].credentials.' + coDwfAbstract.cs_Credential_Key_Repository_v1 + ' = ' + loRepositoryV1 + ' is not supported. ' +
    'Solution: 1). Use a different version of the dwf-deploy module 2). Update the environment of the dwf module');

  // Supported extensions per prioroty
  this.asExt = [
    coTxtLib[coAbstractLib.csExtention],
    coNamespaceLib[coAbstractLib.csExtention],
    coTaskchainLib[coAbstractLib.csExtention]
  ];
  let that = this;
  // Supported extention to Type and checked uniqueness
  this.o_sExt2oType = {};
  let lfPush_o_sExt2oType = function(ioType) {
    let lsExt = ioType[coAbstractLib.csExtention];
    let loType = that.o_sExt2oType[lsExt];
    if (typeof loType === 'undefined') {
      that.o_sExt2oType[lsExt] = ioType;
    } else {
      throw new Error('Failed, because the ' + lsExt + ' extention is already supported by ' +
                      JSON.stringify(loType) + ' and cannot be supported also by ' + JSON.stringify(ioType));
    }
  };
  lfPush_o_sExt2oType(coTxtLib);
  lfPush_o_sExt2oType(coNamespaceLib);
  lfPush_o_sExt2oType(coTaskchainLib);

  // Group the files in the module per extention
  this.o_sExt2asFile = {};
  let lfPush_o_sExt2asFile = function(isExt, isFile) {
    let lasFile_Old = that.o_sExt2asFile[isExt];
    if (typeof lasFile_Old === 'undefined') {
      lasFile_Old = [];
    }
    let lasFile_New = lasFile_Old.concat([isFile]);
    that.o_sExt2asFile[isExt] = lasFile_New;
  };
  this.laoFile.forEach(function(isFile) {
    let lsFile = isFile[0];
    if (coFs.statSync(lsFile).isDirectory()) { // Remove directories
      return;
    }
    let sExtnameLowerCase = coPathLib.extnameNamelessFile(lsFile).toLowerCase(); // Group files
    lfPush_o_sExt2asFile(sExtnameLowerCase, lsFile);
  });

  // Check for unsupported extensions
  let la_sFileUnsupported = [];
  Object.getOwnPropertyNames(this.o_sExt2asFile).forEach(function (isExt) {
    let laoFilePerExt = that.o_sExt2asFile[isExt];
    if (typeof that.o_sExt2oType[isExt] === 'undefined') { // Unsupported
      la_sFileUnsupported = la_sFileUnsupported.concat(laoFilePerExt);
    }
  });
  if (la_sFileUnsupported.length > 0) {
    throw new Error('Failed, because the ' + JSON.stringify(la_sFileUnsupported) + ' files in the module are not supported');
  }

}

Container.prototype.prepare = function() {
  let that = this;
  let lsFunction = coAbstractLib.cfPrepare;
  let loResult = new Result();
  this.asExt.forEach(function(isExt) {
    let loType = that.o_sExt2oType[isExt];
    let lasFile = that.o_sExt2asFile[isExt] || []; // Check - empty src folder, only .txt or .dwfnamespace ot .dwftaskchain
    let lfRef = loType[lsFunction];
    if (typeof lfRef !== 'undefined') {
      let loResultExt = lfRef.apply(that, [lasFile, that.loOption]);
      loResult.addResult(loResultExt);
    }
  });
  return loResult;
};

Container.prototype.createTasks = function(io_sExt2oValue) {
  let that = this;
  let lsFunction = coAbstractLib.cfCreateTasks;
  let loResult = new Result();
  let lsExt_Namespace = coNamespaceLib[coAbstractLib.csExtention];
  let loPreparedValue4Ext = io_sExt2oValue[lsExt_Namespace] || {};
  this.asExt.forEach(function(isExt) {
    let loType = that.o_sExt2oType[isExt];
    let lasFile = that.o_sExt2asFile[isExt] || [];
    let lfRef = loType[lsFunction];
    if (typeof lfRef !== 'undefined') {
      let loResultExt = lfRef.apply(that, [lasFile, loPreparedValue4Ext, that.loCredential, that.loOption]);
      loResult.addResult(loResultExt);
    }
  });
  return loResult;
};

Container.prototype.runTasks = function(io_sExt2afTask, ifCb) {
  let lafTasks = [];

  this.asExt.forEach(function(isExt) {
    let lafTasks4Ext = io_sExt2afTask[isExt];
    if (typeof lafTasks4Ext !== 'undefined') {
      lafTasks = lafTasks.concat(lafTasks4Ext);
    }
  });

  coAsyncLib.series(lafTasks, ifCb);
};

module.exports.Container = Container;
